package net.sergeych.bipack

import kotlinx.serialization.SerialInfo

/**
 * If this annotation is presented in some @Serializable class defition, its instances
 * will be serialized with leadinf number of fields. This allows to extend class later
 * providing new parameters __to the end of the class__ and _with default values__.
 *
 * Whe deserializing such instances from previous version binaries, the new parameters
 * will get default values.
 *
 * Serialized data of classes not market as ExtendableFormat could not be changed without
 * breaking compatibility with existing serialized data.
 */
@Target(AnnotationTarget.CLASS)
@SerialInfo
annotation class ExtendableFormat


/**
 * Serializable classes annotated as Framed will have leading checked mark as CRC32
 * of its name (uses `@SerialName` internally). On deserializing, if frame will not
 * match the expected name, the [InvalidFrameException] will be thrown.
 */
@SerialInfo
@Target(AnnotationTarget.CLASS)
annotation class Framed

@SerialInfo
@Target(AnnotationTarget.CLASS)
annotation class CrcProtected

open class InvalidFrameException(reason: String) : Exception(reason)
class InvalidFrameHeaderException(reason: String = "Frame header does not match") : InvalidFrameException(reason)
class InvalidFrameCRCException : InvalidFrameException("Checksum CRC32 failed")

