package bipack

import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import net.sergeych.bintools.toDump
import net.sergeych.bipack.*
import kotlin.experimental.xor
import kotlin.test.*

@Serializable
data class Foobar1N(val bar: Int, val foo: Int = 117)

@Serializable
@ExtendableFormat
data class Foobar1(val bar: Int, val foo: Int = 117)

@Serializable
@ExtendableFormat
data class Foobar2(val bar: Int, val foo: Int,val other: Int = -1)
@Serializable
@Framed
@ExtendableFormat
data class FoobarF1(val bar: Int, val foo: Int = 117)

@Serializable
@Framed
@ExtendableFormat
@SerialName("bipack.FoobarF1")
data class FoobarF2(val bar: Int, val foo: Int,val other: Int = -1)
@Serializable
@Framed
@ExtendableFormat
data class FoobarF3(val bar: Int, val foo: Int,val other: Int = -1)

@Serializable
@Framed
@ExtendableFormat
@CrcProtected()
data class FoobarFP1(val bar: Int, val foo: Int,val other: Int = -1)

class BipackEncoderTest {

    @Test
    fun encodeSimple() {
        val a = Foobar1(42)//, "bum")
        println(BipackEncoder.encode(a).toDump())
        val b = BipackDecoder.decode<Foobar1>(BipackEncoder.encode(a))
        assertEquals(a, b)
        val c = BipackDecoder.decode<Foobar2>(BipackEncoder.encode(a))
        assertEquals(-1, c.other)
        assertEquals(a.bar, c.bar)
    }
    @Test
    fun encodeFramed() {
        val a = FoobarF1(42)//, "bum")
        println(BipackEncoder.encode(a).toDump())
        val b = BipackDecoder.decode<FoobarF1>(BipackEncoder.encode(a))
        assertEquals(a, b)
        val c = BipackDecoder.decode<FoobarF2>(BipackEncoder.encode(a))
        assertEquals(-1, c.other)
        assertEquals(a.bar, c.bar)
        assertFailsWith(InvalidFrameException::class) {
            BipackDecoder.decode<FoobarF3>(BipackEncoder.encode(a))
        }
    }

    @Test
    fun encodeNonExtendable() {
        val a = Foobar1N(42)//, "bum")
        println(BipackEncoder.encode(a).toDump())
        val b = BipackDecoder.decode<Foobar1N>(BipackEncoder.encode(a))
        assertEquals(a, b)
    }

    @Test
    fun encodeProtectedFrame() {
        val a = FoobarFP1(42, 117)//, "bum")
        println(BipackEncoder.encode(a).toDump())
        val b = BipackDecoder.decode<FoobarFP1>(BipackEncoder.encode(a))
        assertEquals(a, b)
        val bad = BipackEncoder.encode(a)
        bad[6] = bad[6] xor 0x20
        println(bad.toDump())
        assertFailsWith(InvalidFrameCRCException::class) {
            BipackDecoder.decode<FoobarFP1>(bad)
        }
    }

    @Serializable
    data class FBU(val u: UInt, val i: Int)
    @Test
    fun testByteArray() {
        val x = byteArrayOf(1,2,3)
        println(BipackEncoder.encode(x).toDump())
        println(BipackEncoder.encode("123").toDump())
        println(BipackEncoder.encode(FBU(3u, 3)).toDump())
//        println(BipackEncoder.encode(1U).toDump())
    }
}